"""

    """

import shutil
from pathlib import Path

from dulwich import index
from dulwich import porcelain
from dulwich.client import HTTPUnauthorized

from .consts import Consts
from .funcs import clean_github_url
from .funcs import get_github_token_json_fp
from .funcs import get_usr_tok_fr_json_file
from .funcs import github_url_wt_credentials


cte = Consts()


class Repo :

    def __init__(self , source_url , usr_tok_json_fp = None) :
        self.source_url = source_url
        self._ut_jsfp = usr_tok_json_fp

        self.cln_src_url = clean_github_url(source_url)
        self.user_repo = self.cln_src_url.split(cte.gi)[1]
        self.user_name = self.user_repo.split('/')[0]
        self.repo_name = self.user_repo.split('/')[1]

        self._local_path = None

        self._clone_url = None

        self._crd_usr = None
        self._crd_tok = None

        self._commit_url = None

        self._repo = None

        self._init_local_path()

    @property
    def local_path(self) :
        return self._local_path

    @local_path.setter
    def local_path(self , local_dir) :
        if local_dir is None :
            self._local_path = Path(self.repo_name)
        else :
            self._local_path = Path(local_dir)

        if self._local_path.exists() :
            print('Warning: local_path already exists')

    def _init_local_path(self) :
        self.local_path = None

    def _stage_all_changes(self) :
        idx = self._repo.open_index()

        lp = str(self.local_path) + '/'
        fu = porcelain.get_untracked_paths
        untracked = fu(lp , lp , idx , exclude_ignored = True)

        unstaged = index.get_unstaged_changes(idx , self.local_path)

        all_changes = list(unstaged) + list(untracked)

        self._repo.stage(all_changes)
        return all_changes

    def _input_cred_usr_tok(self) :
        usr = input('(enter nothing for same as repo source) github username: ')
        if usr.strip() == '' :
            self._crd_usr = self.user_name
        tok = input('token: ')
        self._crd_tok = tok

    def _set_cred_usr_tok(self) :
        if self._ut_jsfp :
            fp = self._ut_jsfp
            self._crd_usr , self._crd_tok = get_usr_tok_fr_json_file(fp)
            return

        fp = Path('user_token.json')
        if fp.exists() :
            self._crd_usr , self._crd_tok = get_usr_tok_fr_json_file(fp)
            return

        fp = get_github_token_json_fp()
        if fp :
            self._crd_usr , self._crd_tok = get_usr_tok_fr_json_file(fp)
            return

        self._input_cred_usr_tok()

    def _set_clone_url(self) :
        self._set_cred_usr_tok()
        usr = self._crd_usr
        tok = self._crd_tok
        tr = self.user_repo
        self._clone_url = github_url_wt_credentials(usr , tok , tr)

    def overwriting_clone(self , overwrite = True , depth = 1) :
        """ Every time excecuted, it re-downloads last version of the reposiroty to local_path.

        param depth: None for full depth, default = 1 (last version)
        return: None
        """
        trgdir = self.local_path
        if trgdir.exists() and overwrite :
            self.rmdir()

        try :
            self._clone_url = self.cln_src_url
            url = self._clone_url
            self._repo = porcelain.clone(url , trgdir , depth = depth)
        except HTTPUnauthorized :
            self._set_clone_url()
            url = self._clone_url
            self._repo = porcelain.clone(url , trgdir , depth = depth)

    def _set_commit_url(self) :
        if self._clone_url != self.cln_src_url :
            self._commit_url = self._clone_url
        else :
            self._set_cred_usr_tok()
            usr = self._crd_usr
            tok = self._crd_tok
            tr = self.user_repo
            self._commit_url = github_url_wt_credentials(usr , tok , tr)

    def commit_and_push(self , message , branch = 'main') :
        self._set_commit_url()
        self._stage_all_changes()
        self._repo.do_commit(message.encode())
        porcelain.push(str(self._local_path) , self._commit_url , branch)

    def rmdir(self) :
        shutil.rmtree(self._local_path)



"""

    """

import asyncio
import base64
from functools import partial
from pathlib import Path

import aiohttp
import nest_asyncio
import pandas as pd
import ujson
from gidgethub.aiohttp import GitHubAPI

from .funcs import get_token
from .githubb import _get_all_fps_in_repo
from .githubb import ret_pygithub_repo_obj
from .githubb import ret_usr_repo_from_repo_url


nest_asyncio.apply()


def make_header(tok) :
    return {
            "Accept"        : "application/vnd.github.v3+json" ,
            "Authorization" : f"token {tok}" ,
            }


def make_data_json(cont , branch , sha , fn) :
    dta = {
            "content" : cont ,
            "branch"  : branch ,
            }
    if sha :
        dta["sha"] = sha
        dta["message"] = f"{fn} overwritted"
    else :
        dta["message"] = f"{fn} overwritted"
    return dta


async def ret_file_cont_as_base64_encoded_async(fp) :
    with open(fp , 'rb') as f :
        cnt = f.read()
    cnt = base64.b64encode(cnt)
    cnt = cnt.decode('utf-8')
    return cnt


async def add_overwrite_a_file_2_repo_async(fp ,
                                            fn ,
                                            sha ,
                                            usr_repo ,
                                            branch ,
                                            headers) :
    bs64_cnt = await ret_file_cont_as_base64_encoded_async(fp)
    dta = make_data_json(bs64_cnt , branch , sha , fn)
    async with aiohttp.ClientSession(json_serialize = ujson.dumps) as ses :
        url = f"https://api.github.com/repos/{usr_repo}/contents/{fn}"
        async with ses.put(url , headers = headers , json = dta) as r :
            print(await r.text())


async def add_overwrite_files_2_repo_async(fps , repo_url , branch) :
    tok = get_token()

    hdr = make_header(tok)

    ur = ret_usr_repo_from_repo_url(repo_url)

    df = find_sha_of_files(fps , ur , tok)

    fu = partial(add_overwrite_a_file_2_repo_async ,
                 usr_repo = ur ,
                 branch = branch ,
                 headers = hdr)

    co_tsks = []
    for _ , row in df.iterrows() :
        tsk = fu(fp = row['fp'] , fn = row['fn'] , sha = row['sha'])
        co_tsks.append(tsk)

    await asyncio.gather(*co_tsks)


def find_sha_of_files(fps , usr_repo , tok) :
    fps = [Path(fp) for fp in fps]

    data = {
            'fp' : list(fps)
            }
    df = pd.DataFrame(data)

    df['fn'] = df['fp'].apply(lambda x : x.name)

    rp = ret_pygithub_repo_obj(usr_repo , tok)
    ofs = _get_all_fps_in_repo(rp)

    _df = pd.DataFrame(ofs)
    _df['sha'] = _df[0].apply(lambda x : x.sha)
    _df['fn'] = _df[0].apply(lambda x : x.path)

    _df = _df.set_index('fn')
    df['sha'] = df['fn'].map(_df['sha'])
    return df


async def get_rare_limit(requester = 'imahdimir') :
    async with aiohttp.ClientSession() as session :
        gh = GitHubAPI(session , requester , oauth_token = get_token())
        data = await gh.getitem("/rate_limit")
        print(data)



"""

    """

import asyncio
from pathlib import Path

from src.giteasy.funcs import get_token
from src.giteasy.github_async import add_overwrite_a_file_2_repo_async
from src.giteasy.github_async import add_overwrite_files_2_repo_async
from src.giteasy.github_async import get_rare_limit
from src.giteasy.github_async import make_header
from src.giteasy.githubb import ret_usr_repo_from_repo_url


##
fu = get_rare_limit()
asyncio.run(fu)

##
fp = Path('/Users/mahdi/Downloads/Stata_to_LaTeX.pdf')
url = 'https://github.com/imahdimir/td-u-d0-FirmTicker-MonthlySales'

tok = get_token()
hdrs = make_header(tok)
ur = ret_usr_repo_from_repo_url(url)

fu = add_overwrite_a_file_2_repo_async(fp ,
                                       '15.pdf' ,
                                       None ,
                                       ur ,
                                       'main' ,
                                       hdrs)

##
asyncio.run(fu)
##
fu1 = add_overwrite_a_file_2_repo_async(fp ,
                                        '15.pdf' ,
                                        None ,
                                        ur ,
                                        'main' ,
                                        hdrs)

##
_fu = asyncio.gather(*[await fu , await fu1])
asyncio.run(_fu)

##
url = 'https://github.com/imahdimir/td-u-d0-FirmTicker-MonthlySales'

pth = '/Users/mahdi/Dropbox/1-git-dirs/PyCharm/u-d0-FirmTicker-MonthlySales/link-htmls'
diir = Path(pth)

fps = diir.glob('*.html')
fps = list(fps)
fps = fps[:5]

fu = add_overwrite_files_2_repo_async(fps , url , 'main')

asyncio.run(fu)

##
